%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Suren Islyaev
% Main function, contains methods for optimisation
% Inputs:
%     myfunction = pointer to funca iton you want to minimise
%     data = data for minimisation
%     x0 = initial state for optimisation algorithm
%     Precision= number of function Evaluations/Iterations
%     samplesise = size of sample for minimisation
%     testsample = number of data points for out of sample check
% Output:
%     Theta = model parameters
%     RAE = Relative absolute error for each forecast step
%     MRAE = Mean absolute error for commodity forecast 
%     RMSE = ROOT MEAN SQUARED ERROR
%     fval = function value(Log MLE)
%     FutErrors = one step ahead futures price forecast MRAE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Theta,RAE,MRAE,RMSE,fval,FutErrors] = main(myfunction,data, x0, Precision, samplesize,testsample )
[n,m]=size(data);
n_new=samplesize+testsample;
%% Initial values
delta=1/252; %timestep weekly
T=data(n,2:m)*delta; % times to maturity 
x=log(data(1:n_new,1));    %vector of commodity prices price at time t=0
y=log(data(1:n_new,2:m)); %futures prices data 
y=y(~isinf(x),:);   %getting rid of Inf values
x=x(~isinf(x)); %getting rid of Inf values
n=length(x)-1; 
D=T;
%% Define calibration options

    options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'UseParallel','always',...
                     'TolFun',1e-16);
%% start calibration using fminsearch
                     
    [Theta, fval]=fminsearch(@(c) myfunction(c,y(1:...
    samplesize,:),x(1),delta,T,'calibration',D),x0,options);

%% Initialise output
futures=zeros(m-1,n-samplesize-1);
commodity=zeros(n-samplesize-1,1);
Tn=T-samplesize*delta;
k=0;
%% Evaluate one-step ahead forecast
for i=samplesize:n-1
[futures(:,k+1), commodity(k+1)]=myfunction(Theta,y(samplesize+k,:)...
    ,x(samplesize+k),delta,Tn,'test',D);
k=k+1;
end;
 %% Set the output values
 realComm=exp(x(samplesize+1:n));
 comcum=exp(commodity(1:end));
 X=realComm-comcum;
 N=length(X);
 RAE=abs(X)./realComm*100;
 MRAE=mean(RAE);
 RMSE=sqrt(sum((X).^2)/N);
 FutErrors=mean(abs(y(samplesize+1:n,:)'-futures(:,:))./y(samplesize+1:n,:)',2)';
end

