%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function twostatefilter
% Author: Suren Islyaev
% Kalman filter used for 2 state process
% Input:
%     c=vector of parameters
%     y=futures price data
%     x0=initial state of the filter
%     delta = timestep
%     T=time to maturity
%     tag= method used(calibration or test for one step ahead forecast)
% Output:
%     z=estimated futures price one step ahead vector or MLE depends on tag
%     value
%     commodity = estimated commodity price one step ahaed value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [z, commodity]=twostatefilter(c,y,x0,delta,T,tag,~)
[m, n]=size(y);
%%%% Following parameter transformations were implemented to avoid getting
%%%% wrong parameter valued during optimisation, because Nelder-Mead does
%%%% unconstrained optimisation
a2=c(1);            %alpha_2
k=abs(c(2))+0.01;   %kappa
sigma1=abs(c(3));   %sigma_1 abs used to avoid negative values
sigma2=abs(c(4));   %sigma_2
Q=eye(n)*(abs(c(5))+0.01);  % measurement noise matrix
QQ=Q^2; 
l1=c(6);            % risk premium for the process one
l2=c(7);            % risk premium for the process two
rho=sin(c(8));      % correlation between processes
p=[abs(c(9))+0.0001 0; 0 abs(c(10))+0.0001];    % covariance between states
expkd=exp(-k*delta);        % e^{k\Delat}
%Initialise vectors and matrices%
x_old=[0 zeros(1,m);x0 zeros(1,m)];
x_new=[zeros(1,m+1);zeros(1,m+1)];
%For more information see the paper%
B=[expkd 0; 0 1];
RR=[sigma1^2*0.5/k*(1-expkd^2) rho*sigma1*sigma2/k*(1-expkd);...
      rho*sigma1*sigma2/k*(1-expkd) sigma2^2*delta];
g=[0;a2*delta];
Af=@(t) [exp(-k*t) ones(length(t),1)];
Tv=repmat(T',1,m+1)-repmat(0:m,n,1)*delta;
d=l1*sigma1/k*(1-exp(-k*Tv))+(a2+l2*sigma2)*Tv+...
    sigma1^2*0.25/k*(1-exp(-2*k*Tv))+...
    rho*sigma1*sigma2/k*(1-exp(-k*Tv))+0.5*sigma2^2*Tv;

y=y';
switch lower(tag)
    case {'calibration'}
        z=0;
        for i=2:m-1
            A=Af(Tv(:,i+1));            % Update matrix A
            x_new(:,i-1)=B*x_old(:,i-1)+g;  % state estimate x_{n|n+1}
            v=y(:,i)-(A*(x_new(:,i-1))+d(:,i)); %innovation
            sig=A*p*A'+QQ;                      % innovation covariance
            k=p*A'/sig;                         % Kalman gain
            x_old(:,i)=x_new(:,i-1)+k*v;        %posteriory update
            p=B*p*B'+RR-B*p*A'*(sig\A)*p*B';    %posteriory covariance update
            z=z+log(det(sig))+v'*(sig\v);       % summ up MLE
        end;
    
    case {'test'}
            commodity=B*x_old(:,1)+g;       % state estimate apriori
            z=Af(Tv(:,2))*(commodity)+d(:,2);   %futures state estimate apriori
            commodity=sum(commodity);       % sum of the states as commodity estimate
end;        
        
        